require 'asbru/components/savage'

module Asbru
  # This module defines a bunch of methods that
  module CML
    C = Asbru::Components::Savage

    # Problems:
    # How can we make this module work with Sage/Savage depending on config?
    # Do we want to do that?

    # Ideas:
    # 1. We can make Sage implement to exactly the same interface as Savage and
    #  somehow inject it.
    # 2. We could also ALWAYS load the sage classed needed here and call these
    # required.
    #
    # I think I prefer 1 even though the syntax of Sage is nicer imo.
    class << self
      def p text, **opts
        C.send "#{component_prefix}_content",
            text: text,
            **opts
      end

      def h1 text, **opts
        C.send "#{component_prefix}_title",
            size: 'h1',
            text: text
      end

      def h2 text, **opts
        C.send "#{component_prefix}_title",
            size: 'h2',
            text: text
      end

      def h3 text, **opts
        C.send "#{component_prefix}_title",
            size: 'h3',
            text: text
      end

      def h4 text, **opts
        C.send "#{component_prefix}_title",
            size: 'h4',
            text: text
      end

      def h5 text, **opts
        C.send "#{component_prefix}_title",
            size: 'h5',
            text: text
      end

      def hr color: 'stable-500'
        C.send "#{component_prefix}_divider",
            color: color,
            height: 1
      end

      def vertical_spacer height
        C.send "#{component_prefix}_divider",
            color: :transparent,
            height: height
      end

      def table_row(*cells)
        { data: cells }
      end

      def table_cell(text, action: nil, method: nil, confirm: nil, color: nil)
        cell = { text: text}
        cell[:color] = color if color.present?
        cell[:action] = action if action.present?
        cell[:customData] = {}
        cell[:customData]['data-method'] = method if method.present?
        cell[:customData]['data-confirm'] = confirm if confirm.present?
        cell
      end

      private

      def component_prefix
        Asbru.config.component_prefix
      end
    end
  end
end
